@@ -58,6 +58,15 @@ class AwkwardArray(awkward.util.NDArrayOperatorsMixin):
 
     allow_tonumpy = True
     allow_iter = True
+    # TODO for 1.0: add check_prop_valid and check_whole_valid parameters
+
+    DEFAULTTYPE = awkward.util.DEFAULTTYPE
+    CHARTYPE    = awkward.util.CHARTYPE
+    INDEXTYPE   = awkward.util.INDEXTYPE
+    TAGTYPE     = awkward.util.TAGTYPE
+    MASKTYPE    = awkward.util.MASKTYPE
+    BITMASKTYPE = awkward.util.BITMASKTYPE
+    BOOLTYPE    = awkward.util.BOOLTYPE
 
     def _checktonumpy(self):
         if not self.allow_tonumpy:
@@ -105,7 +114,7 @@ class AwkwardArray(awkward.util.NDArrayOperatorsMixin):
 
     @property
     def type(self):
-        return awkward.type.ArrayType(*(self._getshape() + (awkward.type._resolve(self._gettype({}), {}),)))
+        return awkward.type.ArrayType(len(self), awkward.type._resolve(self._gettype({}), {}))
 
     @property
     def dtype(self):
