@@ -91,7 +91,7 @@ class Table(awkward.array.base.AwkwardArray):
                 return self._table.Row(table, index + where)
 
         def __dir__(self):
-            return ["_" + x for x in self._table._content if x.isnumeric() or x.isidentifier()] + ["tolist"]
+            return ["at", "tolist"]
 
         def __iter__(self, checkiter=True):
             if checkiter:
@@ -288,7 +288,7 @@ class Table(awkward.array.base.AwkwardArray):
         if not isinstance(value, dict) or not all(isinstance(n, awkward.util.string) for n in value):
             raise TypeError("content must be a dict from strings to arrays")
         for n in list(value):
-            value[n] = awkward.util.toarray(value[n], awkward.util.DEFAULTTYPE)
+            value[n] = awkward.util.toarray(value[n], self.DEFAULTTYPE)
         self._content = value
 
     def __len__(self):
@@ -300,9 +300,6 @@ class Table(awkward.array.base.AwkwardArray):
             out[n] = awkward.type._fromarray(x, seen)
         return out
 
-    def _getshape(self):
-        return (self._length(),)
-
     def _length(self):
         if self._view is None:
             if len(self._content) == 0:
@@ -385,7 +382,7 @@ class Table(awkward.array.base.AwkwardArray):
                 return self._view[head]
 
         else:
-            head = awkward.util.toarray(head, awkward.util.INDEXTYPE)
+            head = awkward.util.toarray(head, self.INDEXTYPE)
             if issubclass(head.dtype.type, awkward.util.numpy.integer):
                 length = self._length()
                 negative = (head < 0)
@@ -452,7 +449,7 @@ class Table(awkward.array.base.AwkwardArray):
                 index = self._index()
                 try:
                     if index is None:
-                        return self._content[where]
+                        return self._content[where][:self._length()]
                     else:
                         return self._content[where][index]
                 except KeyError:
@@ -473,7 +470,7 @@ class Table(awkward.array.base.AwkwardArray):
         head, tail = where[0], where[1:]
 
         if tail != ():
-            raise IndexError("cannot pass multidimensional indexes through a Table")
+            raise NotImplementedError("multidimensional index through a Table (TODO: needed for [0, n) -> [0, m) -> \"table\" -> ...)")
 
         newslice = self._newslice(head)
 
@@ -491,13 +488,13 @@ class Table(awkward.array.base.AwkwardArray):
             raise ValueError("new columns can only be attached to the original Table, not a view (try table.base['col'] = array)")
 
         if isinstance(where, awkward.util.string):
-            self._content[where] = awkward.util.toarray(what, awkward.util.DEFAULTTYPE)
+            self._content[where] = awkward.util.toarray(what, self.DEFAULTTYPE)
 
         elif awkward.util.isstringslice(where):
             if len(where) != len(what):
                 raise ValueError("number of keys ({0}) does not match number of provided arrays ({1})".format(len(where), len(what)))
             for x, y in zip(where, what):
-                self._content[x] = awkward.util.toarray(y, awkward.util.DEFAULTTYPE)
+                self._content[x] = awkward.util.toarray(y, self.DEFAULTTYPE)
 
         else:
             raise TypeError("invalid index for assigning column to Table: {0}".format(where))
